@@ -1,5 +1,5 @@
 /* i387-specific utility functions, for the remote server for GDB.
-   Copyright (C) 2000, 2001, 2002, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2000-2014 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -19,12 +19,15 @@
 #ifndef I387_FP_H
 #define I387_FP_H
 
-void i387_cache_to_fsave (void *buf);
-void i387_fsave_to_cache (const void *buf);
+void i387_cache_to_fsave (struct regcache *regcache, void *buf);
+void i387_fsave_to_cache (struct regcache *regcache, const void *buf);
 
-void i387_cache_to_fxsave (void *buf);
-void i387_fxsave_to_cache (const void *buf);
+void i387_cache_to_fxsave (struct regcache *regcache, void *buf);
+void i387_fxsave_to_cache (struct regcache *regcache, const void *buf);
 
-extern int num_xmm_registers;
+void i387_cache_to_xsave (struct regcache *regcache, void *buf);
+void i387_xsave_to_cache (struct regcache *regcache, const void *buf);
+
+extern unsigned long long x86_xcr0;
 
 #endif /* I387_FP_H */
