@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2007, 2008 Free Software Foundation, Inc.
+   Copyright 2007-2014 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,10 +26,15 @@ main (void)
   }
 
 #else
+void
+bar (void)
+  {
+  }
+
 void
 foo (void)
   {
-    /* Break here  */
+    bar(); /* Break here  */
   }
 
 #endif
